﻿using System;
using System.IO;
using System.Xml.Serialization;

namespace Demo06_OverwriteXmlSerializer
{
    internal class Program
    {
        static void Main(string[] args)
        {
            Program test = new Program();
            test.SerializeObject("Override.xml");
            test.DeserializeObject("Override.xml");
        }
        public void SerializeObject(string filename)
        {
            
            XmlAttributes attrs = new XmlAttributes();

            /* Create an XmlElementAttribute to override the
            field that returns Instrument objects. The overridden field
            returns Brass objects instead. */
            //创建XmlElementAttribute以覆盖返回仪器对象的字段。替代的字段返回黄铜对象。
            XmlElementAttribute attr = new XmlElementAttribute();
            attr.ElementName = "Brass";
            attr.Type = typeof(Brass);

            // 添加元素到集合
            attrs.XmlElements.Add(attr);

            // 创建XmlAttributeOverrides
            XmlAttributeOverrides attrOverrides = new XmlAttributeOverrides();

            /* Add the type of the class that contains the overridden
            member and the XmlAttributes to override it with to the
            XmlAttributeOverrides object. */
            //将包含重写成员的类的类型以及要用其重写的XmlAttributeOverrides对象的XmlAttributes添加到XmlAttributeOverrides对象。
            attrOverrides.Add(typeof(Orchestra), "Instruments", attrs);

            // Create the XmlSerializer using the XmlAttributeOverrides.
            XmlSerializer s = new XmlSerializer(typeof(Orchestra), attrOverrides);

            using (TextWriter writer = new StreamWriter(filename)) 
            {
                // Create the object that will be serialized.
                Orchestra band = new Orchestra();

                // Create an object of the derived type.
                Brass i = new Brass();
                i.Name = "Trumpet";
                i.IsValved = true;
                Instrument[] myInstruments = { i };
                band.Instruments = myInstruments;

                // Serialize the object.
                s.Serialize(writer, band);
            }

            
        }

        public void DeserializeObject(string filename)
        {
            XmlAttributeOverrides attrOverrides =
               new XmlAttributeOverrides();
            XmlAttributes attrs = new XmlAttributes();

            // Create an XmlElementAttribute to override the Instrument.
            XmlElementAttribute attr = new XmlElementAttribute();
            attr.ElementName = "Brass";
            attr.Type = typeof(Brass);

            // Add the XmlElementAttribute to the collection of objects.
            attrs.XmlElements.Add(attr);

            attrOverrides.Add(typeof(Orchestra), "Instruments", attrs);

            // Create the XmlSerializer using the XmlAttributeOverrides.
            XmlSerializer s =
            new XmlSerializer(typeof(Orchestra), attrOverrides);

            FileStream fs = new FileStream(filename, FileMode.Open);
            Orchestra band = (Orchestra)s.Deserialize(fs);
            Console.WriteLine("Brass:");

            /* The difference between deserializing the overridden
            XML document and serializing it is this: To read the derived
            object values, you must declare an object of the derived type
            (Brass), and cast the Instrument instance to it. */
            Brass b;
            foreach (Instrument i in band.Instruments)
            {
                b = (Brass)i;
                Console.WriteLine(
                b.Name + "\n" +
                b.IsValved);
            }
        }
    }
}
